<?php

namespace Drupal\cforge\Plugin\Field\FieldFormatter;

use Drupal\cforge\Plugin\Field\FieldType\ContactMeItem;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\user\UserData;
use Drupal\Core\Session\AccountProxy;

/**
 * Plugin implementation of the 'author' formatter.
 *
 * @FieldFormatter(
 *   id = "cf_contactme",
 *   label = @Translation("Contact me channels"),
 *   description = @Translation("Display emails, phone, url and social media handles."),
 *   field_types = {
 *     "cf_contactme"
 *   }
 * )
 */
class ContactMeFormatter extends FormatterBase  {

  /**
   * @var AccountProxy
   */
  private $currentUser;

  /**
   * @var UserData;
   */
  private $userData;

  /**
   *
   * @param type $plugin_id
   * @param type $plugin_definition
   * @param type $field_definition
   * @param type $settings
   * @param type $label
   * @param type $view_mode
   * @param type $third_party_settings
   * @param UserData $user_data
   * @param AccountProxy $current_user
   */
  function __construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings, UserData $user_data, AccountProxy $current_user) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->currentUser = $current_user;
    $this->userData = $user_data;
  }

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static (
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('user.data'),
      $container->get('current_user')
    );
  }

   /**
   * {@inheritDoc}
   */
  public static function defaultSettings() {
    return [
      'channels' => [],
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    return [
      'channels' => [
        '#title' => $this->t('Channels to render'),
        '#type' => 'checkboxes',
        '#default_value' => $this->getSetting('channels'),
        '#options' => ContactMeItem::channelOptions()
      ]
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $enabled = array_intersect_key(ContactMeItem::channelOptions(), $this->settings['channels']);
    return implode(', ', $enabled);
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    if (\Drupal::currentUser()->isAuthenticated()) {
      // requires a lot of scaffolding code to inject into formatters.
      $privacy = $this->userData->get('cforge', $items->getEntity()->id());

      $i = 0;
      // Put the definitive email if settings permit.
      if (!empty($privacy['show_email'])) {
        $elements[$i++] = [
          '#markup' => "<div class = \"channels email\">".$this->t('Email (definitive)').': '.$items->getEntity()->getEmail().'</div>',
          '#access' => \Drupal::service('user.data')->get('cforge', $items->getEntity()->id(), 'show_email'),
          '#weight' => $i
        ];
      }
      $available_channels = array_filter($items->getFieldDefinition()->getSetting('channels'));
      if (empty($privacy['show_phones'])) {
        unset($available_channels['tel'], $available_channels['mob'], $available_channels['whatsapp']);
      }
      foreach ($available_channels as $channel) {
        if (empty($this->settings['channels']) or $this->settings['channels'][$channel]) {
          // Todo we should only show the properties enabled in widget settings
          if ($val = $items->{$channel}) {
            $title = ContactMeItem::allChannels()[$channel]['title'];
            $elements[$i++] = [
              '#markup' => '<div class = "channels '.$channel.'" title = "'.$title.'"> '.$items->{$channel}.'</div>',
              '#weight' => $i
            ];
          }
        }
      }
      if ($elements) {
        $elements = [
          '#title' => t('Contact details'),
          '#type' => 'details',
          '#open' => FALSE,
          '#attached' => ['library'=> ['cforge/social_media_icons']],
        ] + $elements;
      }
    }
    return $elements;
  }

  /**
   * {@inheritDoc}
   */
  function prepareView(array $entities_items) {
    parent::prepareView($entities_items);
    $all_channels = ContactMeItem::allChannels();
    foreach ($entities_items as $item) {
      if ($item->altmail) {
        $url = $all_channels['altmail']['prefix'] . $item->altmail;
        $item->altmail = '<a href="'.$url.'">'.$item->altmail.'</a>';
      }
      if ($item->whatsapp) { //  Must come before mob and tel
        $number = $item->{$item->whatsapp};
        $url = $all_channels['whatsapp']['prefix'] . preg_replace('/^[0-9]/', '', $number);
        $item->whatsapp = '<a href="'.$url.'">'.$number.'</a>';
      }
      if ($item->tel) {
        if ($this->cansee($item->getEntity(), 'tel')) {
          $url = $all_channels['tel']['prefix'] . preg_replace('/^[0-9]/', '', $item->tel);
          $item->tel = '<a href="'.$url.'">'.$item->tel.'</a>';
        }
        else {
          $item->tel = '';
        }
      }
      if ($item->mob) {
        $url = $all_channels['mob']['prefix'] . preg_replace('/^[0-9]/', '', $item->mob);
        $item->mob = '<a href="'.$url.'">'.$item->mob.'</a>';
      }
      if ($item->web) {
        $url = $all_channels['web']['prefix'] . $item->web;
        $item->web = '<a href="'.$url.'">'.$item->web.'</a>';
      }
      if ($item->twitter) {
        $url = $all_channels['twitter']['prefix'] . $item->twitter;
        $item->twitter = '<a href="'.$url.'">'.$item->twitter.'</a>';
      }
      if ($item->telegram) {
        $url = $all_channels['telegram']['prefix'] . $item->telegram;
        $item->telegram = '<a href="'.$url.'">'.$item->telegram.'</a>';
      }
      if ($item->instagram) {
        $url = $all_channels['instagram']['prefix'] . $item->instagram;
        $item->instagram = '<a href="'.$url.'">'.$item->instagram.'</a>';
      }
      if ($item->linkedin) {
        $url = $all_channels['linkedin']['prefix'] . $item->linkedin;
        $item->linkedin = '<a href="'.$url.'">'.$item->linkedin.'</a>';
      }
      if ($item->skype) {
        $url = $all_channels['skype']['prefix'] . $item->skype . '?chat';
        $item->skype = '<a href="'.$url.'">'.$item->skype.'</a>';
      }
      if ($item->facebook) {
        $url = $all_channels['facebook']['prefix'] . $item->facebook;
        $item->facebook = '<a href="'.$url.'">'.$item->facebook.'</a>';
      }
    }
  }

  /**
   * Check the user's privacy settings on the given property.
   * @param type $property
   */
  function canSee($owner, $property) : bool {
    if ($this->currentUser->hasPermission('administer users') or $this->currentUser->id() == $owner->id()) {
      return TRUE;
    }
    if ($this->currentUser->isAnonymous()) {
      return FALSE;
    }
    switch($property) {
      case 'tel':
      case 'mob':
        $key = 'show_phones';
        break;
      case 'email':
        $key = 'show_email'; break;
    }
    $stored = $this->userData->get('cforge', $owner->id(), $key);
    return (bool)$stored;
  }

}
